package main

import (
	"bytes"
	"io/ioutil"
	"os"
	"path"
	"path/filepath"
	"strings"

	"gopkg.in/yaml.v3"
)

func collectStructureFromFiles(directory string) (error, map[string]*requirement) {

	cleanedDirectory, err := filepath.Abs(path.Clean(directory))

	pageMap := make(map[string]*requirement)

	err = filepath.Walk(cleanedDirectory,
		func(current string, info os.FileInfo, err error) error {
			if err != nil {
				return err
			}

			cleanedPath := path.Clean(current)

			if info.IsDir() {
				return nil
			}

			ext := filepath.Ext(cleanedPath)
			if ext != ".md" {
				return nil
			}

			content, err := ioutil.ReadFile(cleanedPath)

			if err != nil {
				return err
			}

			err, p := splitYamlParts(content)

			if err != nil {
				printWarning("the file contains structural errors. check the YAML and if necessary the template.", cleanedPath, err)
			}

			bd := []byte(cleanedDirectory)
			bp := []byte(cleanedPath)
			bs := string(bp[len(bd):len(bp)])

			p.data.File = "." + bs
			p.data.Absolute = cleanedPath
			p.data.ToDos = findTask(content)

			k := cleanedPath

			pageMap[k] = &p.data

			return nil
		})

	return err, pageMap
}

func rewriteFiles(pageData map[string]*requirement) error {

	for filename, pd := range pageData {

		buf := new(bytes.Buffer)

		var encoder = yaml.NewEncoder(buf)
		err := encoder.Encode(pd.OriginNode)
		if err != nil {
			return err
		}
		encoder.Close()

		text := "---\n" + buf.String() + "\n...\n"
		text += strings.Trim(pd.OriginText, "\n")

		info, err := os.Stat(filename)
		if err != nil {
			return err
		}
		mode := info.Mode()

		err = os.WriteFile(filename, []byte(text), mode)
		if err != nil {
			return err
		}

	}

	return nil
}
